# coding: utf-8
from rest_framework.request import Request
from datetime import datetime, timedelta
from inspect_report.models import Tasks, CheckSession, RulesStat, SeatStat
from config.config import TABLE_PRE
import json
import logging
"""定时任务

参考：https://github.com/jgorset/django-kronos
"""
logger = logging.getLogger("app_file")


def rule_stat(start_date, end_date):
    """
    首页概述-违规项统计
    :return:
    """
    task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date}
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    stat_count = 0
    for t in tasks:
        if RulesStat.objects.filter(taskId=t['id']).exists():
            continue
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id'], 'violationRuleCount__gt': 0}
        checks = tn.objects.filter(**session_condition).values('id', 'checkResult', 'agentName', 'customName', 'score',
                                                               'remainTime', 'taskId', 'sessionId')
        rules_list = []
        for check in checks:
            result = check['checkResult']
            if result:
                data = json.loads(result)
                for d in data:
                    if 'isViolation' in d.keys() and d['isViolation'] and 'rule' in d.keys() and 'name' in d['rule'].keys():
                        name = d['rule']['name']
                        rule_obj = {'create_date': start_date, 'sessionCollectionId': t['sessionCollectionId'],
                                    'rule': name, 'rule_num': 1, 'task': t['name'].split('_')[0],
                                    'agentName': check['agentName'], 'customName': check['customName'],
                                    'score': check['score'], 'remainTime': check['remainTime'],
                                    'taskId': check['taskId'], 'sessionId': check['sessionId'],
                                    'session_table_id': check['id']}
                        rules_list.append(RulesStat(**rule_obj))
                        stat_count += 1
                        logger.info('统计数量为：%s' % stat_count)
        RulesStat.objects.bulk_create(rules_list)


def seat_stat(start_date, end_date):
    """
    首页概述-违规项统计
    :return:
    """

    task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date}
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    for t in tasks:
        if SeatStat.objects.filter(taskId=t['id']).exists():
            continue
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id']}
        checks = tn.objects.filter(**session_condition).values('id', 'agentName', 'taskId', 'violationRuleCount')
        seat_dict = {}
        seat_list = []
        for check in checks:
            if check['agentName'] in seat_dict.keys():
                seat = seat_dict.get(check['agentName'])
                seat['total_session'] += 1
                seat['validate_session'] += 1 if check['violationRuleCount'] > 0 else 0
            else:
                seat_dict[check['agentName']] = {'create_date': start_date, 'agentName': check['agentName'],
                                                 'taskId': check['taskId'], 'task': t['name'].split('_')[0],
                                                 'sessionCollectionId': t['sessionCollectionId'], 'total_session': 1}
                seat_dict[check['agentName']]['validate_session'] = 1 if check['violationRuleCount'] > 0 else 0
        for k, v in seat_dict.items():
            seat_obj = {'agentName': k}
            seat_obj.update(v)
            seat_list.append(SeatStat(**seat_obj))
        SeatStat.objects.bulk_create(seat_list)
